@@ -15,12 +15,12 @@
 """Helpers for :mod:`grpc`."""
 
 import collections
+import functools
+import warnings
 
 import grpc
-import six
 
 from google.api_core import exceptions
-from google.api_core import general_helpers
 import google.auth
 import google.auth.credentials
 import google.auth.transport.grpc
@@ -29,10 +29,18 @@ import google.auth.transport.requests
 try:
     import grpc_gcp
 
+    warnings.warn(
+        """Support for grpcio-gcp is deprecated. This feature will be
+        removed from `google-api-core` after January 1, 2024. If you need to
+        continue to use this feature, please pin to a specific version of
+        `google-api-core`.""",
+        DeprecationWarning,
+    )
     HAS_GRPC_GCP = True
 except ImportError:
     HAS_GRPC_GCP = False
 
+
 # The list of gRPC Callable interfaces that return iterators.
 _STREAM_WRAP_CLASSES = (grpc.UnaryStreamMultiCallable, grpc.StreamStreamMultiCallable)
 
@@ -51,25 +59,26 @@ def _wrap_unary_errors(callable_):
     """Map errors for Unary-Unary and Stream-Unary gRPC callables."""
     _patch_callable_name(callable_)
 
-    @six.wraps(callable_)
+    @functools.wraps(callable_)
     def error_remapped_callable(*args, **kwargs):
         try:
             return callable_(*args, **kwargs)
         except grpc.RpcError as exc:
-            six.raise_from(exceptions.from_grpc_error(exc), exc)
+            raise exceptions.from_grpc_error(exc) from exc
 
     return error_remapped_callable
 
 
 class _StreamingResponseIterator(grpc.Call):
-    def __init__(self, wrapped):
+    def __init__(self, wrapped, prefetch_first_result=True):
         self._wrapped = wrapped
 
         # This iterator is used in a retry context, and returned outside after init.
         # gRPC will not throw an exception until the stream is consumed, so we need
         # to retrieve the first result, in order to fail, in order to trigger a retry.
         try:
-            self._stored_first_result = six.next(self._wrapped)
+            if prefetch_first_result:
+                self._stored_first_result = next(self._wrapped)
         except TypeError:
             # It is possible the wrapped method isn't an iterable (a grpc.Call
             # for instance). If this happens don't store the first result.
@@ -82,7 +91,7 @@ class _StreamingResponseIterator(grpc.Call):
         """This iterator is also an iterable that returns itself."""
         return self
 
-    def next(self):
+    def __next__(self):
         """Get the next response from the stream.
 
         Returns:
@@ -93,13 +102,10 @@ class _StreamingResponseIterator(grpc.Call):
                 result = self._stored_first_result
                 del self._stored_first_result
                 return result
-            return six.next(self._wrapped)
+            return next(self._wrapped)
         except grpc.RpcError as exc:
             # If the stream has already returned data, we cannot recover here.
-            six.raise_from(exceptions.from_grpc_error(exc), exc)
-
-    # Alias needed for Python 2/3 support.
-    __next__ = next
+            raise exceptions.from_grpc_error(exc) from exc
 
     # grpc.Call & grpc.RpcContext interface
 
@@ -137,13 +143,20 @@ def _wrap_stream_errors(callable_):
     """
     _patch_callable_name(callable_)
 
-    @general_helpers.wraps(callable_)
+    @functools.wraps(callable_)
     def error_remapped_callable(*args, **kwargs):
         try:
             result = callable_(*args, **kwargs)
-            return _StreamingResponseIterator(result)
+            # Auto-fetching the first result causes PubSub client's streaming pull
+            # to hang when re-opening the stream, thus we need examine the hacky
+            # hidden flag to see if pre-fetching is disabled.
+            # https://github.com/googleapis/python-pubsub/issues/93#issuecomment-630762257
+            prefetch_first = getattr(callable_, "_prefetch_first_result_", True)
+            return _StreamingResponseIterator(
+                result, prefetch_first_result=prefetch_first
+            )
         except grpc.RpcError as exc:
-            six.raise_from(exceptions.from_grpc_error(exc), exc)
+            raise exceptions.from_grpc_error(exc) from exc
 
     return error_remapped_callable
 
@@ -170,34 +183,71 @@ def wrap_errors(callable_):
         return _wrap_unary_errors(callable_)
 
 
-def _create_composite_credentials(credentials=None, scopes=None, ssl_credentials=None):
+def _create_composite_credentials(
+    credentials=None,
+    credentials_file=None,
+    default_scopes=None,
+    scopes=None,
+    ssl_credentials=None,
+    quota_project_id=None,
+    default_host=None,
+):
     """Create the composite credentials for secure channels.
 
     Args:
         credentials (google.auth.credentials.Credentials): The credentials. If
             not specified, then this function will attempt to ascertain the
             credentials from the environment using :func:`google.auth.default`.
+        credentials_file (str): A file with credentials that can be loaded with
+            :func:`google.auth.load_credentials_from_file`. This argument is
+            mutually exclusive with credentials.
+        default_scopes (Sequence[str]): A optional list of scopes needed for this
+            service. These are only used when credentials are not specified and
+            are passed to :func:`google.auth.default`.
         scopes (Sequence[str]): A optional list of scopes needed for this
             service. These are only used when credentials are not specified and
             are passed to :func:`google.auth.default`.
         ssl_credentials (grpc.ChannelCredentials): Optional SSL channel
             credentials. This can be used to specify different certificates.
+        quota_project_id (str): An optional project to use for billing and quota.
+        default_host (str): The default endpoint. e.g., "pubsub.googleapis.com".
 
     Returns:
         grpc.ChannelCredentials: The composed channel credentials object.
+
+    Raises:
+        google.api_core.DuplicateCredentialArgs: If both a credentials object and credentials_file are passed.
     """
-    if credentials is None:
-        credentials, _ = google.auth.default(scopes=scopes)
-    else:
-        credentials = google.auth.credentials.with_scopes_if_required(
-            credentials, scopes
+    if credentials and credentials_file:
+        raise exceptions.DuplicateCredentialArgs(
+            "'credentials' and 'credentials_file' are mutually exclusive."
         )
 
+    if credentials_file:
+        credentials, _ = google.auth.load_credentials_from_file(
+            credentials_file, scopes=scopes, default_scopes=default_scopes
+        )
+    elif credentials:
+        credentials = google.auth.credentials.with_scopes_if_required(
+            credentials, scopes=scopes, default_scopes=default_scopes
+        )
+    else:
+        credentials, _ = google.auth.default(
+            scopes=scopes, default_scopes=default_scopes
+        )
+
+    if quota_project_id and isinstance(
+        credentials, google.auth.credentials.CredentialsWithQuotaProject
+    ):
+        credentials = credentials.with_quota_project(quota_project_id)
+
     request = google.auth.transport.requests.Request()
 
     # Create the metadata plugin for inserting the authorization header.
     metadata_plugin = google.auth.transport.grpc.AuthMetadataPlugin(
-        credentials, request
+        credentials,
+        request,
+        default_host=default_host,
     )
 
     # Create a set of grpc.CallCredentials using the metadata plugin.
@@ -207,12 +257,20 @@ def _create_composite_credentials(credentials=None, scopes=None, ssl_credentials
         ssl_credentials = grpc.ssl_channel_credentials()
 
     # Combine the ssl credentials and the authorization credentials.
-    return grpc.composite_channel_credentials(
-        ssl_credentials, google_auth_credentials
-    )
+    return grpc.composite_channel_credentials(ssl_credentials, google_auth_credentials)
 
 
-def create_channel(target, credentials=None, scopes=None, ssl_credentials=None, **kwargs):
+def create_channel(
+    target,
+    credentials=None,
+    scopes=None,
+    ssl_credentials=None,
+    credentials_file=None,
+    quota_project_id=None,
+    default_scopes=None,
+    default_host=None,
+    **kwargs
+):
     """Create a secure channel with credentials.
 
     Args:
@@ -225,22 +283,37 @@ def create_channel(target, credentials=None, scopes=None, ssl_credentials=None,
             are passed to :func:`google.auth.default`.
         ssl_credentials (grpc.ChannelCredentials): Optional SSL channel
             credentials. This can be used to specify different certificates.
+        credentials_file (str): A file with credentials that can be loaded with
+            :func:`google.auth.load_credentials_from_file`. This argument is
+            mutually exclusive with credentials.
+        quota_project_id (str): An optional project to use for billing and quota.
+        default_scopes (Sequence[str]): Default scopes passed by a Google client
+            library. Use 'scopes' for user-defined scopes.
+        default_host (str): The default endpoint. e.g., "pubsub.googleapis.com".
         kwargs: Additional key-word args passed to
             :func:`grpc_gcp.secure_channel` or :func:`grpc.secure_channel`.
+            Note: `grpc_gcp` is only supported in environments with protobuf < 4.0.0.
 
     Returns:
         grpc.Channel: The created channel.
+
+    Raises:
+        google.api_core.DuplicateCredentialArgs: If both a credentials object and credentials_file are passed.
     """
+
     composite_credentials = _create_composite_credentials(
-        credentials, scopes, ssl_credentials
+        credentials=credentials,
+        credentials_file=credentials_file,
+        default_scopes=default_scopes,
+        scopes=scopes,
+        ssl_credentials=ssl_credentials,
+        quota_project_id=quota_project_id,
+        default_host=default_host,
     )
 
     if HAS_GRPC_GCP:
-        # If grpc_gcp module is available use grpc_gcp.secure_channel,
-        # otherwise, use grpc.secure_channel to create grpc channel.
         return grpc_gcp.secure_channel(target, composite_credentials, **kwargs)
-    else:
-        return grpc.secure_channel(target, composite_credentials, **kwargs)
+    return grpc.secure_channel(target, composite_credentials, **kwargs)
 
 
 _MethodCall = collections.namedtuple(
